package com.vodori.cms.component.navigation.tag;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyContent;
import javax.servlet.jsp.tagext.BodyTagSupport;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author Ben Fagin (Vodori)
 * @version 1/19/11
 */
public class ColorFilterTag extends BodyTagSupport {
	private static Pattern hexPattern = Pattern.compile("#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})");
	private int red, blue, green, alpha;

	public void setRed(Integer red) {
		this.red = red == null ? 0 : red;
	}

	public void setBlue(Integer blue) {
		this.blue = blue == null ? 0 : blue;
	}

	public void setGreen(Integer green) {
		this.green = green == null ? 0 : green;
	}

	public void setAlpha(Integer alpha) {
		this.alpha = alpha == null ? 0 : alpha;
	}

	@Override
	public int doAfterBody() throws JspException {
		BodyContent bc = getBodyContent();
		String body = bc.getString();
		bc.clearBody();

		// find all color hashes and modify them
		Matcher matcher = hexPattern.matcher(body);
		StringBuilder sb = new StringBuilder();
		int start = 0;

		while (matcher.find()) {
			sb.append(body.substring(start, matcher.start()));
			String hex = body.substring(matcher.start(), matcher.end());
			sb.append(modifyColor(hex));
			start = matcher.end();
		} sb.append(body.substring(start));


		try {
			getPreviousOut().print(sb.toString());
		} catch (Exception ex) {
			throw new JspException("Uh oh!");
		}

		// don't need to re-evaluate the body
		return SKIP_BODY;
	}

	private String modifyColor(String hex) {
		int val = Integer.decode(hex);
		int a = (val >> 24) & 0xFF;
		int g = (val >> 16) & 0xFF;
		int b = (val >> 8) & 0xFF;
		int r = val & 0xFF;

		// modify according to instance settings
		r = r + red;	
		if (r > 255)
			r = 255;
		else if (r < 0)
			r = 0;

		g = g + green;	
		if (g > 255)
			g = 255;
		else if (g < 0)
			g = 0;
		
		b = b + blue;
		if (b > 255)
			b = 255;
		else if (b < 0)
			b = 0;
		
		a = a + alpha;	
		if (a > 255)
			a = 255;
		else if (a < 0)
			a = 0;

		val = (a << 24) |
			  (g << 16) |
			  (b << 8) |
			  r;

		return "#" + Integer.toHexString(val);
	}
}
