package com.vodori.cms.component.navigation.tag;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.*;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Ben Fagin (Vodori)
 * @version 1/19/11
 */
public class ReflectiveTag extends BodyTagSupport {
	private TagSupport _basicHandler;
	private BodyTagSupport _bodyHandler;
	private Map<String, Object> _attributes;
	private boolean _initialized = false;

	// convenience params
	private String _attr1K, _attr2K, _attr3K;
	private Object _attr1V, _attr2V, _attr3V;

	/**
	 * Handler must implement BodyTag, which itself extends IterationTag.
	 * @param handler
	 */
	public void setHandler(String handler) {
		instantiateHandler(handler);
	}

	public void setAttributes(Map attributes) {
		_attributes = attributes;
	}

	// convenience methods for up to three attributes
	public void setAttr1K(String attr) {
		_attr1K = attr;
	}

	public void setAttr2K(String attr) {
		_attr2K = attr;
	}

	public void setAttr3K(String attr) {
		_attr3K = attr;
	}

	public void setAttr1V(Object attr) {
		_attr1V = attr;
	}

	public void setAttr2V(Object attr) {
		_attr2V = attr;
	}

	public void setAttr3V(Object attr) {
		_attr3V = attr;
	}


	/**
	 * Reflection.
	 * 
	 * @param clazz
	 * @return
	 */
	private void instantiateHandler(String clazz) {
		try {
			Object obj = Class.forName(clazz).newInstance();

			if (obj instanceof BodyTagSupport) {
				_bodyHandler = (BodyTagSupport) obj;
				_basicHandler = _bodyHandler;
			} else if (obj instanceof TagSupport) {
				_basicHandler = (TagSupport) obj;
			}

		} catch (Exception ex) {
			throw new RuntimeException(ex);
			//TODO or something better
			//or just return null and let an NPE occur
		}
	}

	/**
	 * set all of the handler's attributes where possible.
	 */
	private void setHandlerAttributes() {
		Class clazz = _basicHandler.getClass();
		
		for (Map.Entry<String, Object> entry : _attributes.entrySet()) {
			try {
				String methodName = Character.toUpperCase(entry.getKey().charAt(0)) + entry.getKey().substring(1);
				methodName = "set" + methodName;
				Class[] types = new Class[] { entry.getValue().getClass() };
				Method method = clazz.getMethod(methodName, types);
				method.invoke(_basicHandler, entry.getValue());
			} catch (Exception ex) {
				; //nothing
			}
		}
	}

	private void injectHeldValues() {
		_basicHandler.setPageContext(_heldContext);
		_basicHandler.setParent(_heldTag);
	}

	@Override
	public int doStartTag() throws JspException {
		if (!_initialized) {
			if (_attributes == null) {
				_attributes = new HashMap<String, Object>();
			}

			// put convenience methods into place
			String keys[] = new String[]{_attr1K, _attr2K, _attr3K};
			Object values[] = new Object[]{_attr1V, _attr2V, _attr3V};

			for (int i=0; i < keys.length; ++i) {
				if (keys[i] != null) {
					_attributes.put(keys[i], values[i]);
				}
			}

			injectHeldValues();
			setHandlerAttributes();
			_initialized = true;
		}
		
		return _basicHandler.doStartTag();
	}

	@Override
	public int doEndTag() throws JspException {
		return _basicHandler.doEndTag();
	}

	@Override
	public void setBodyContent(BodyContent b) {
		if (_bodyHandler != null)
			_bodyHandler.setBodyContent(b);
		else
			super.setBodyContent(b);
	}

	@Override
	public void doInitBody() throws JspException {
		if (_bodyHandler != null)
			_bodyHandler.doInitBody();
		else
			super.doInitBody();
	}

	@Override
	public int doAfterBody() throws JspException {
		return _basicHandler.doAfterBody();
	}

	@Override
	public void release() {
		_basicHandler.release();
	}

	@Override
	public BodyContent getBodyContent() {
		if (_bodyHandler != null)
			return _bodyHandler.getBodyContent();
		else
			return super.getBodyContent();
	}

	@Override
	public JspWriter getPreviousOut() {
		if (_bodyHandler != null)
			return _bodyHandler.getPreviousOut();
		else
			return super.getPreviousOut();
	}

	@Override
	public void setId(String id) {
		_basicHandler.setId(id);
	}

	@Override
	public String getId() {
		return _basicHandler.getId();
	}

	@Override
	public void setValue(String k, Object o) {
		_basicHandler.setValue(k, o);
	}

	@Override
	public Object getValue(String k) {
		return _basicHandler.getValue(k);
	}

	@Override
	public void removeValue(String k) {
		_basicHandler.removeValue(k);
	}

	@Override
	public Enumeration getValues() {
		return _basicHandler.getValues();
	}

	@Override
	public int hashCode() {
		return _basicHandler.hashCode();
	}

	@Override
	public boolean equals(Object obj) {
		return _basicHandler.equals(obj);
	}

	@Override
	public String toString() {
		return _basicHandler.toString();
	}


	/*
		Handler isn't set before these get called, so store their values for later.
	 */
	private Tag _heldTag;
	private PageContext _heldContext;

	@Override
	public void setParent(Tag t) {
		_heldTag = t;
	}

	@Override
	public Tag getParent() {
		return _basicHandler.getParent();
	}

	@Override
	public void setPageContext(PageContext pageContext) {
		_heldContext = pageContext;
	}
}
